//Minesweeper - Bill Marty's Java learning exercise.
//Duplicate the MS Windows game Minesweeper.
//Yes, I know Minesweeper is Copyrighted by Microsoft.  My intent is not to
// sell or distribute, only to learn Java GUI programming.
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;

public class MyMines extends JApplet {

   //The Applet, which is basically the start button.
   Container appletPane;
   JButton startButton;
   Border raisedBorder;
   ImageIcon bigSmile;
   URL codeBase;

   //The Frame, which is the game board.
   JFrame game;
   Container gamePane;
   JMenuBar menuBar;
   GameMenu gameMenu;
   HelpMenu helpMenu;
   Smiley smiley;          //The smiley face button that starts a new game.
   MineCnt unmarkedMines;  //The counter for unmarked mines.
   LEDtime seconds;        //Elapsed time since the first move.
   Sea mySea;              //The grid of sea cells that forms the playing area.
   DebugText debugText;    //Optional area for writing debug messages.

   boolean firstGame = true;
   boolean activeGame = false;

   public void init() {
      appletPane = getContentPane();
      codeBase = getCodeBase();
      bigSmile = new ImageIcon(getImage(codeBase, "bigSmile.gif"));
      startButton = new JButton("Play Minesweeper", bigSmile);
      //startButton size is set by Applet dimensions in html file.
      startButton.setDefaultCapable(false);
      raisedBorder = BorderFactory.createBevelBorder(BevelBorder.RAISED);
      startButton.setBorder(raisedBorder);
      startButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            if(!activeGame) {
               activeGame = true;
               runGame();
            }
         }
      });
      appletPane.add(startButton);
   }

   class WindowCloseListener extends WindowAdapter {
      public void windowClosed(WindowEvent e) {
         activeGame = false;
      }
   }

   void runGame() {
      game = new JFrame("Java Demo");
      game.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
      Game.setBeginner();
      game.setSize(Game.width, Game.height);
      game.setResizable(false);
      game.setLocationRelativeTo(null);   //Center game on screen.
      menuBar = new JMenuBar();
      gameMenu = new GameMenu(this);
      helpMenu = new HelpMenu(this);
      smiley = new Smiley(this);
      unmarkedMines = new MineCnt(this);
      seconds = new LEDtime(this);
      debugText = new DebugText();

      gamePane = game.getContentPane();
      gamePane.setLayout(new FlowLayout());
      //Build the menu bar.
      menuBar.add(gameMenu);
      menuBar.add(helpMenu);

      game.setJMenuBar(menuBar);
      gamePane.add(unmarkedMines.getJPanel());  //Component 0
      gamePane.add(smiley);                     //Component 1
      gamePane.add(seconds.getJPanel());        //Component 2
      setupNewGame(false);                      //Installs Component 3
      if(debugText.isDebugEnabled()) {          //Component 4
         gamePane.add(debugText.getDebugPane());
      }

      game.addWindowListener(new WindowCloseListener());
      game.setVisible(true);
   }

   void setupNewGame(boolean sizeChange) {
      Game.started = false;
      Game.over = false;
      unmarkedMines.initialize();
      seconds.reset();
      debugText.reset();
      //Do we need to remove an existing mySea?
      if(firstGame) {
         firstGame = false;
         mySea = new Sea(this, debugText, unmarkedMines, smiley);
         gamePane.add(mySea.getJPanel(),3);
      }
      else {
         gamePane.remove(mySea.getJPanel());
         mySea = new Sea(this, debugText, unmarkedMines, smiley);
         smiley.setSmile();
         gamePane.add(mySea.getJPanel(),3);
         if(sizeChange) {
            game.setSize(new Dimension(Game.width,Game.height));
            game.setLocationRelativeTo(null);   //Center game on screen.
         }
         game.setVisible(true);              //Force frame to repaint.
      }
   }//End setupNewGame.

}//End MyMines.
